@@ -67,7 +67,7 @@ void mutt_refresh (void)
 }
 
 /* Make sure that the next refresh does a full refresh.  This could be
-   optmized by not doing it at all if DISPLAY is set as this might
+   optimized by not doing it at all if DISPLAY is set as this might
    indicate that a GUI based pinentry was used.  Having an option to
    customize this is of course the Mutt way.  */
 void mutt_need_hard_redraw (void)
@@ -98,7 +98,10 @@ event_t mutt_getch (void)
   mutt_allow_interrupt (0);
 
   if (SigInt)
+  {
     mutt_query_exit ();
+    return err;
+  }
 
   if(ch == ERR)
   {
@@ -126,7 +129,7 @@ event_t mutt_getch (void)
   return (ch == ctrl ('G') ? err : ret);
 }
 
-int _mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int complete, int multiple, char ***files, int *numfiles)
+int _mutt_get_field (const char *field, char *buf, size_t buflen, int complete, int multiple, char ***files, int *numfiles)
 {
   int ret;
   int x, y;
@@ -136,7 +139,7 @@ int _mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int comp
   do
   {
     CLEARLINE (LINES-1);
-    addstr (field);
+    addstr ((char *)field); /* cast to get around bad prototypes */
     mutt_refresh ();
     getyx (stdscr, y, x);
     ret = _mutt_enter_string (buf, buflen, y, x, complete, multiple, files, numfiles, es);
@@ -192,6 +195,7 @@ int mutt_yesorno (const char *msg, int def)
   char *no = _("no");
   char *answer_string;
   size_t answer_string_len;
+  size_t msglen;
 
 #ifdef HAVE_LANGINFO_YESEXPR
   char *expr;
@@ -204,9 +208,9 @@ int mutt_yesorno (const char *msg, int def)
   answer[1] = 0;
   
   reyes_ok = (expr = nl_langinfo (YESEXPR)) && expr[0] == '^' &&
-	     !regcomp (&reyes, expr, REG_NOSUB|REG_EXTENDED);
+	     !REGCOMP (&reyes, expr, REG_NOSUB);
   reno_ok = (expr = nl_langinfo (NOEXPR)) && expr[0] == '^' &&
-            !regcomp (&reno, expr, REG_NOSUB|REG_EXTENDED);
+            !REGCOMP (&reno, expr, REG_NOSUB);
 #endif
 
   CLEARLINE(LINES-1);
@@ -217,10 +221,12 @@ int mutt_yesorno (const char *msg, int def)
    * ensure there is enough room for the answer and truncate the question
    * to fit.
    */
-  answer_string = safe_malloc (COLS + 1);
-  snprintf (answer_string, COLS + 1, " ([%s]/%s): ", def == M_YES ? yes : no, def == M_YES ? no : yes);
-  answer_string_len = strlen (answer_string);
-  printw ("%.*s%s", COLS - answer_string_len, msg, answer_string);
+  safe_asprintf (&answer_string, " ([%s]/%s): ", def == M_YES ? yes : no, def == M_YES ? no : yes);
+  answer_string_len = mutt_strwidth (answer_string);
+  /* maxlen here is sort of arbitrary, so pick a reasonable upper bound */
+  msglen = mutt_wstr_trunc (msg, 4*COLS, COLS - answer_string_len, NULL);
+  addnstr (msg, msglen);
+  addstr (answer_string);
   FREE (&answer_string);
 
   FOREVER
@@ -275,6 +281,12 @@ int mutt_yesorno (const char *msg, int def)
     addstr ((char *) (def == M_YES ? yes : no));
     mutt_refresh ();
   }
+  else
+  {
+    /* when the users cancels with ^G, clear the message stored with
+     * mutt_message() so it isn't displayed when the screen is refreshed. */
+    mutt_clear_error();
+  }
   return (def);
 }
 
@@ -295,54 +307,49 @@ void mutt_query_exit (void)
   SigInt = 0;
 }
 
-void mutt_curses_error (const char *fmt, ...)
+static void curses_message (int error, const char *fmt, va_list ap)
 {
-  va_list ap;
   char scratch[LONG_STRING];
 
-  va_start (ap, fmt);
   vsnprintf (scratch, sizeof (scratch), fmt, ap);
-  va_end (ap);
-  
+
   dprint (1, (debugfile, "%s\n", scratch));
   mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, FMT_LEFT, 0, scratch, sizeof (scratch), 0);
+		      0, COLS, FMT_LEFT, 0, scratch, sizeof (scratch), 0);
 
   if (!option (OPTKEEPQUIET))
   {
-    BEEP ();
-    SETCOLOR (MT_COLOR_ERROR);
+    if (error)
+      BEEP ();
+    SETCOLOR (error ? MT_COLOR_ERROR : MT_COLOR_MESSAGE);
     mvaddstr (LINES-1, 0, Errorbuf);
+    NORMAL_COLOR;
     clrtoeol ();
-    SETCOLOR (MT_COLOR_NORMAL);
     mutt_refresh ();
   }
 
-  set_option (OPTMSGERR);
+  if (error)
+    set_option (OPTMSGERR);
+  else
+    unset_option (OPTMSGERR);
+}
+
+void mutt_curses_error (const char *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  curses_message (1, fmt, ap);
+  va_end (ap);
 }
 
 void mutt_curses_message (const char *fmt, ...)
 {
   va_list ap;
-  char scratch[LONG_STRING];
 
   va_start (ap, fmt);
-  vsnprintf (scratch, sizeof (scratch), fmt, ap);
+  curses_message (0, fmt, ap);
   va_end (ap);
-
-  mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, FMT_LEFT, 0, scratch, sizeof (scratch), 0);
-
-  if (!option (OPTKEEPQUIET))
-  {
-    SETCOLOR (MT_COLOR_MESSAGE);
-    mvaddstr (LINES - 1, 0, Errorbuf);
-    clrtoeol ();
-    SETCOLOR (MT_COLOR_NORMAL);
-    mutt_refresh ();
-  }
-
-  unset_option (OPTMSGERR);
 }
 
 void mutt_progress_init (progress_t* progress, const char *msg,
@@ -460,9 +467,9 @@ void mutt_show_error (void)
     return;
   
   SETCOLOR (option (OPTMSGERR) ? MT_COLOR_ERROR : MT_COLOR_MESSAGE);
-  CLEARLINE (LINES-1);
-  addstr (Errorbuf);
-  SETCOLOR (MT_COLOR_NORMAL);
+  mvaddstr(LINES-1, 0, Errorbuf);
+  NORMAL_COLOR;
+  clrtoeol();
 }
 
 void mutt_endwin (const char *msg)
@@ -471,10 +478,10 @@ void mutt_endwin (const char *msg)
 
   if (!option (OPTNOCURSES))
   {
-    CLEARLINE (LINES - 1);
-    
-    attrset (A_NORMAL);
-    mutt_refresh ();
+    /* at least in some situations (screen + xterm under SuSE11/12) endwin()
+     * doesn't properly flush the screen without an explicit call.
+     */
+    mutt_refresh();
     endwin ();
   }
   
@@ -730,6 +737,11 @@ void mutt_format_string (char *dest, size_t destlen,
       w = 1; /* hack */
     else
     {
+#ifdef HAVE_ISWBLANK
+      if (iswblank (wc))
+	wc = ' ';
+      else
+#endif
       if (!IsWPrint (wc))
 	wc = '?';
       w = wcwidth (wc);
@@ -741,7 +753,7 @@ void mutt_format_string (char *dest, size_t destlen,
       min_width -= w;
       max_width -= w;
       strncpy (p, scratch, k2);
-      p += k2;            
+      p += k2;
       destlen -= k2;
     }
   }
@@ -880,7 +892,7 @@ int mutt_wstr_trunc (const char *src, size_t maxlen, size_t maxwid, size_t *widt
 {
   wchar_t wc;
   int w = 0, l = 0, cl;
-  size_t cw, n;
+  int cw, n;
   mbstate_t mbstate;
 
   if (!src)
@@ -894,7 +906,13 @@ int mutt_wstr_trunc (const char *src, size_t maxlen, size_t maxwid, size_t *widt
     if (cl == (size_t)(-1) || cl == (size_t)(-2))
       cw = cl = 1;
     else
+    {
       cw = wcwidth (wc);
+      /* hack because M_TREE symbols aren't turned into characters
+       * until rendered by print_enriched_string (#3364) */
+      if (cw < 0 && cl == 1 && src[0] && src[0] < M_TREE_MAX)
+	cw = 1;
+    }
     if (cl + l > maxlen || cw + w > maxwid)
       break;
     l += cl;
