@@ -47,29 +47,31 @@
 #include "md5.h"
 #include "rfc822.h"
 
+unsigned int hcachever = 0x0;
+
 #if HAVE_QDBM
-static struct header_cache
+struct header_cache
 {
   VILLA *db;
   char *folder;
   unsigned int crc;
-} HEADER_CACHE;
+};
 #elif HAVE_TC
-static struct header_cache
+struct header_cache
 {
   TCBDB *db;
   char *folder;
   unsigned int crc;
-} HEADER_CACHE;
+};
 #elif HAVE_GDBM
-static struct header_cache
+struct header_cache
 {
   GDBM_FILE db;
   char *folder;
   unsigned int crc;
-} HEADER_CACHE;
+};
 #elif HAVE_DB4
-static struct header_cache
+struct header_cache
 {
   DB_ENV *env;
   DB *db;
@@ -77,7 +79,7 @@ static struct header_cache
   unsigned int crc;
   int fd;
   char lockfile[_POSIX_PATH_MAX];
-} HEADER_CACHE;
+};
 
 static void mutt_hcache_dbt_init(DBT * dbt, void *data, size_t len);
 static void mutt_hcache_dbt_empty_init(DBT * dbt);
@@ -586,10 +588,11 @@ mutt_hcache_per_folder(const char *path, const char *folder,
 }
 
 /* This function transforms a header into a char so that it is useable by
- * db_store */
+ * db_store.
+ */
 static void *
 mutt_hcache_dump(header_cache_t *h, HEADER * header, int *off,
-		 unsigned int uidvalidity)
+		 unsigned int uidvalidity, mutt_hcache_store_flags_t flags)
 {
   unsigned char *d = NULL;
   HEADER nh;
@@ -598,14 +601,14 @@ mutt_hcache_dump(header_cache_t *h, HEADER * header, int *off,
   *off = 0;
   d = lazy_malloc(sizeof (validate));
 
-  if (uidvalidity)
-    memcpy(d, &uidvalidity, sizeof (uidvalidity));
-  else
+  if (flags & M_GENERATE_UIDVALIDITY)
   {
     struct timeval now;
     gettimeofday(&now, NULL);
     memcpy(d, &now, sizeof (struct timeval));
   }
+  else
+    memcpy(d, &uidvalidity, sizeof (uidvalidity));
   *off += sizeof (validate);
 
   d = dump_int(h->crc, d, off);
@@ -757,10 +760,17 @@ mutt_hcache_fetch_raw (header_cache_t *h, const char *filename,
 #endif
 }
 
+/*
+ * flags
+ *
+ * M_GENERATE_UIDVALIDITY
+ * ignore uidvalidity param and store gettimeofday() as the value
+ */
 int
 mutt_hcache_store(header_cache_t *h, const char *filename, HEADER * header,
 		  unsigned int uidvalidity,
-		  size_t(*keylen) (const char *fn))
+		  size_t(*keylen) (const char *fn),
+		  mutt_hcache_store_flags_t flags)
 {
   char* data;
   int dlen;
@@ -769,7 +779,7 @@ mutt_hcache_store(header_cache_t *h, const char *filename, HEADER * header,
   if (!h)
     return -1;
   
-  data = mutt_hcache_dump(h, header, &dlen, uidvalidity);
+  data = mutt_hcache_dump(h, header, &dlen, uidvalidity, flags);
   ret = mutt_hcache_store_raw (h, filename, data, dlen, keylen);
   
   FREE(&data);
@@ -1105,7 +1115,7 @@ mutt_hcache_delete(header_cache_t *h, const char *filename,
 header_cache_t *
 mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
 {
-  struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
+  struct header_cache *h = safe_calloc(1, sizeof (struct header_cache));
   int (*hcache_open) (struct header_cache* h, const char* path);
   struct stat sb;
 
@@ -1119,9 +1129,41 @@ mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
   hcache_open = hcache_open_db4;
 #endif
 
+  /* Calculate the current hcache version from dynamic configuration */
+  if (hcachever == 0x0) {
+    unsigned char digest[16];
+    struct md5_ctx ctx;
+    SPAM_LIST *spam;
+    RX_LIST *nospam;
+
+    hcachever = HCACHEVER;
+
+    md5_init_ctx(&ctx);
+
+    /* Seed with the compiled-in header structure hash */
+    md5_process_bytes(&hcachever, sizeof(hcachever), &ctx);
+
+    /* Mix in user's spam list */
+    for (spam = SpamList; spam; spam = spam->next)
+    {
+      md5_process_bytes(spam->rx->pattern, strlen(spam->rx->pattern), &ctx);
+      md5_process_bytes(spam->template, strlen(spam->template), &ctx);
+    }
+
+    /* Mix in user's nospam list */
+    for (nospam = NoSpamList; nospam; nospam = nospam->next)
+    {
+      md5_process_bytes(nospam->rx->pattern, strlen(nospam->rx->pattern), &ctx);
+    }
+
+    /* Get a hash and take its bytes as an (unsigned int) hash version */
+    md5_finish_ctx(&ctx, digest);
+    hcachever = *((unsigned int *)digest);
+  }
+
   h->db = NULL;
   h->folder = get_foldername(folder);
-  h->crc = HCACHEVER;
+  h->crc = hcachever;
 
   if (!path || path[0] == '\0')
   {
