@@ -52,8 +52,8 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   pattern_t *pat = NULL;
   char path[_POSIX_PATH_MAX];
 
-  memset (&pattern, 0, sizeof (pattern));
-  memset (&command, 0, sizeof (command));
+  mutt_buffer_init (&pattern);
+  mutt_buffer_init (&command);
 
   if (*s->dptr == '!')
   {
@@ -279,13 +279,13 @@ void mutt_folder_hook (char *path)
 {
   HOOK *tmp = Hooks;
   BUFFER err, token;
-  char buf[STRING];
 
   current_hook_type = M_FOLDERHOOK;
-  
-  err.data = buf;
-  err.dsize = sizeof (buf);
-  memset (&token, 0, sizeof (token));
+
+  mutt_buffer_init (&err);
+  err.dsize = STRING;
+  err.data = safe_malloc (err.dsize);
+  mutt_buffer_init (&token);
   for (; tmp; tmp = tmp->next)
   {
     if(!tmp->command)
@@ -301,12 +301,15 @@ void mutt_folder_hook (char *path)
 	  FREE (&token.data);
 	  mutt_sleep (1);	/* pause a moment to let the user see the error */
 	  current_hook_type = 0;
+	  FREE (&err.data);
+
 	  return;
 	}
       }
     }
   }
   FREE (&token.data);
+  FREE (&err.data);
   
   current_hook_type = 0;
 }
@@ -328,13 +331,13 @@ void mutt_message_hook (CONTEXT *ctx, HEADER *hdr, int type)
 {
   BUFFER err, token;
   HOOK *hook;
-  char buf[STRING];
 
   current_hook_type = type;
-  
-  err.data = buf;
-  err.dsize = sizeof (buf);
-  memset (&token, 0, sizeof (token));
+
+  mutt_buffer_init (&err);
+  err.dsize = STRING;
+  err.data = safe_malloc (err.dsize);
+  mutt_buffer_init (&token);
   for (hook = Hooks; hook; hook = hook->next)
   {
     if(!hook->command)
@@ -348,10 +351,14 @@ void mutt_message_hook (CONTEXT *ctx, HEADER *hdr, int type)
 	  mutt_error ("%s", err.data);
 	  mutt_sleep (1);
 	  current_hook_type = 0;
+	  FREE (&err.data);
+
 	  return;
 	}
   }
   FREE (&token.data);
+  FREE (&err.data);
+
   current_hook_type = 0;
 }
 
@@ -467,14 +474,14 @@ void mutt_account_hook (const char* url)
   HOOK* hook;
   BUFFER token;
   BUFFER err;
-  char buf[STRING];
 
   if (inhook)
     return;
 
-  err.data = buf;
-  err.dsize = sizeof (buf);
-  memset (&token, 0, sizeof (token));
+  mutt_buffer_init (&err);
+  err.dsize = STRING;
+  err.data = safe_malloc (err.dsize);
+  mutt_buffer_init (&token);
 
   for (hook = Hooks; hook; hook = hook->next)
   {
@@ -489,6 +496,7 @@ void mutt_account_hook (const char* url)
       {
 	FREE (&token.data);
 	mutt_error ("%s", err.data);
+	FREE (&err.data);
 	mutt_sleep (1);
 
         inhook = 0;
@@ -500,5 +508,6 @@ void mutt_account_hook (const char* url)
   }
 
   FREE (&token.data);
+  FREE (&err.data);
 }
 #endif
