@@ -30,7 +30,6 @@
 
 #include "mutt.h"
 #include "imap_private.h"
-#include "message.h"
 #include "mx.h"
 
 #ifdef HAVE_PGP
@@ -65,13 +64,13 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   char *hdrreq = NULL;
   FILE *fp;
   char tempfile[_POSIX_PATH_MAX];
-  int msgno, idx;
+  int msgno, idx = msgbegin - 1;
   IMAP_HEADER h;
   IMAP_STATUS* status;
   int rc, mfhrc, oldmsgcount;
   int fetchlast = 0;
   int maxuid = 0;
-  const char *want_headers = "DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE CONTENT-DESCRIPTION IN-REPLY-TO REPLY-TO LINES LIST-POST X-LABEL";
+  static const char * const want_headers = "DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE CONTENT-DESCRIPTION IN-REPLY-TO REPLY-TO LINES LIST-POST X-LABEL";
   progress_t progress;
   int retval = -1;
 
@@ -160,8 +159,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         rc = imap_cmd_step (idata);
         if (rc != IMAP_CMD_CONTINUE)
 	{
-          /* suppress GCC aliasing warning */
-	  imap_free_header_data ((void**) (void*) &h.data);
+	  imap_free_header_data (&h.data);
           break;
 	}
 
@@ -173,7 +171,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
           continue;
         else if (mfhrc < 0)
 	{
-	  imap_free_header_data ((void**) (void*) &h.data);
+	  imap_free_header_data (&h.data);
           break;
 	}
 
@@ -185,7 +183,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
           continue;
         }
 
-        idx = h.sid - 1;
+        idx++;
         ctx->hdrs[idx] = imap_hcache_get (idata, h.data->uid);
         if (ctx->hdrs[idx])
         {
@@ -209,8 +207,9 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         {
 	  /* bad header in the cache, we'll have to refetch. */
           dprint (3, (debugfile, "bad cache entry at %d, giving up\n", h.sid - 1));
-          imap_free_header_data((void**) (void*) &h.data);
+          imap_free_header_data(&h.data);
           evalhc = 0;
+          idx--;
         }
       }
       while (rc != IMAP_CMD_OK && mfhrc == -1);
@@ -218,8 +217,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         break;
       if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
       {
-        if (h.data)
-          imap_free_header_data ((void**) (void*) &h.data);
+        imap_free_header_data (&h.data);
         imap_hcache_close (idata);
 	goto error_out_1;
       }
@@ -273,18 +271,20 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       {
         dprint (2, (debugfile, "msg_fetch_header: ignoring fetch response with no body\n"));
         mfhrc = -1;
+        msgend--;
         continue;
       }
 
       /* make sure we don't get remnants from older larger message headers */
       fputs ("\n\n", fp);
 
-      idx = h.sid - 1;
+      idx++;
       if (idx > msgend)
       {
         dprint (1, (debugfile, "imap_read_headers: skipping FETCH response for "
                     "unknown message number %d\n", h.sid));
         mfhrc = -1;
+        idx--;
         continue;
       }
       /* May receive FLAGS updates in a separate untagged response (#2935) */
@@ -292,6 +292,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       {
 	dprint (2, (debugfile, "imap_read_headers: message %d is not new\n",
 		    h.sid));
+        idx--;
 	continue;
       }
 
@@ -333,8 +334,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
 
     if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
     {
-      if (h.data)
-        imap_free_header_data ((void**) (void*) &h.data);
+      imap_free_header_data (&h.data);
 #if USE_HCACHE
       imap_hcache_close (idata);
 #endif
@@ -770,8 +770,8 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   /* loop in case of TRYCREATE */
   do
   {
-    memset (&sync_cmd, 0, sizeof (sync_cmd));
-    memset (&cmd, 0, sizeof (cmd));
+    mutt_buffer_init (&sync_cmd);
+    mutt_buffer_init (&cmd);
 
     /* Null HEADER* means copy tagged messages */
     if (!h)
@@ -1015,12 +1015,14 @@ void imap_add_keywords (char* s, HEADER* h, LIST* mailbox_flags, size_t slen)
 }
 
 /* imap_free_header_data: free IMAP_HEADER structure */
-void imap_free_header_data (void** data)
+void imap_free_header_data (IMAP_HEADER_DATA** data)
 {
-  /* this should be safe even if the list wasn't used */
-  mutt_free_list (&(((IMAP_HEADER_DATA*) *data)->keywords));
-
-  FREE (data);		/* __FREE_CHECKED__ */
+  if (*data)
+  {
+    /* this should be safe even if the list wasn't used */
+    mutt_free_list (&((*data)->keywords));
+    FREE (data); /* __FREE_CHECKED__ */
+  }
 }
 
 /* imap_set_flags: fill out the message header according to the flags from
