@@ -27,7 +27,6 @@
 #include "url.h"
 #include "imap_private.h"
 #ifdef USE_HCACHE
-#include "message.h"
 #include "hcache.h"
 #endif
 
@@ -146,7 +145,7 @@ int imap_hcache_put (IMAP_DATA* idata, HEADER* h)
 
   sprintf (key, "/%u", HEADER_DATA (h)->uid);
   return mutt_hcache_store (idata->hcache, key, h, idata->uid_validity,
-                            imap_hcache_keylen);
+                            imap_hcache_keylen, 0);
 }
 
 int imap_hcache_del (IMAP_DATA* idata, unsigned int uid)
@@ -162,7 +161,7 @@ int imap_hcache_del (IMAP_DATA* idata, unsigned int uid)
 #endif
 
 /* imap_parse_path: given an IMAP mailbox name, return host, port
- *   and a path IMAP servers will recognise.
+ *   and a path IMAP servers will recognize.
  * mx.mbox is malloc'd, caller must free it */
 int imap_parse_path (const char* path, IMAP_MBOX* mx)
 {
@@ -374,7 +373,7 @@ IMAP_DATA* imap_new_idata (void)
   if (!idata)
     return NULL;
 
-  if (!(idata->cmdbuf = mutt_buffer_init (NULL)))
+  if (!(idata->cmdbuf = mutt_buffer_new ()))
     FREE (&idata);
 
   idata->cmdslots = ImapPipelineDepth + 2;
@@ -612,7 +611,8 @@ void imap_qualify_path (char *dest, size_t len, IMAP_MBOX *mx, char* path)
  *   surround string with quotes, escape " and \ with \ */
 void imap_quote_string (char *dest, size_t dlen, const char *src)
 {
-  char quote[] = "\"\\", *pt;
+  static const char quote[] = "\"\\";
+  char *pt;
   const char *s;
 
   pt = dest;
@@ -733,7 +733,7 @@ int imap_wordcasecmp(const char *a, const char *b)
  *
  */
 
-static RETSIGTYPE alrm_handler (int sig)
+static void alrm_handler (int sig)
 {
   /* empty */
 }
