@@ -36,7 +36,7 @@
 
 #include "functions.h"
 
-struct mapping_t Menus[] = {
+const struct mapping_t Menus[] = {
  { "alias",	MENU_ALIAS },
  { "attach",	MENU_ATTACH },
  { "browser",	MENU_FOLDER },
@@ -167,16 +167,17 @@ static int parse_fkey(char *s)
  */
 static int parse_keycode (const char *s)
 {
-    const char *endChar;
-    long int result = strtol(s+1, &endChar, 8);
-    /* allow trailing whitespace, eg.  < 1001 > */
-    while (ISSPACE(*endChar))
-	++endChar;
-    /* negative keycodes don't make sense, also detect overflow */
-    if (*endChar != '>' || result < 0 || result == LONG_MAX) {
-	return -1;
-    }
-    return result;
+  char *endChar;
+  long int result = strtol(s+1, &endChar, 8);
+  /* allow trailing whitespace, eg.  < 1001 > */
+  while (ISSPACE(*endChar))
+    ++endChar;
+  /* negative keycodes don't make sense, also detect overflow */
+  if (*endChar != '>' || result < 0 || result == LONG_MAX) {
+    return -1;
+  }
+
+  return result;
 }
 
 static int parsekeys (const char *str, keycode_t *d, int max)
@@ -297,7 +298,7 @@ void km_bindkey (char *s, int menu, int op)
   km_bind (s, menu, op, NULL, NULL);
 }
 
-static int get_op (struct binding_t *bindings, const char *start, size_t len)
+static int get_op (const struct binding_t *bindings, const char *start, size_t len)
 {
   int i;
 
@@ -311,7 +312,7 @@ static int get_op (struct binding_t *bindings, const char *start, size_t len)
   return OP_NULL;
 }
 
-static char *get_func (struct binding_t *bindings, int op)
+static char *get_func (const struct binding_t *bindings, int op)
 {
   int i;
 
@@ -365,7 +366,7 @@ static void push_string (char *s)
 	 * skip the '<' and the '>' when comparing */
 	for (i = 0; Menus[i].name; i++)
 	{
-	  struct binding_t *binding = km_get_table (Menus[i].value);
+	  const struct binding_t *binding = km_get_table (Menus[i].value);
 	  if (binding)
 	  {
 	    op = get_op (binding, pp + 1, l - 2);
@@ -407,7 +408,7 @@ static int retry_generic (int menu, keycode_t *keys, int keyslen, int lastkey)
 /* return values:
  *	>0		function to execute
  *	OP_NULL		no function bound to key sequence
- *	-1		error occured while reading input
+ *	-1		error occurred while reading input
  */
 int km_dokey (int menu)
 {
@@ -435,8 +436,11 @@ int km_dokey (int menu)
 	  timeout (ImapKeepalive * 1000);
 	  tmp = mutt_getch ();
 	  timeout (-1);
-	  if (tmp.ch != -2)
-	    /* something other than timeout */
+	  /* If a timeout was not received, or the window was resized, exit the
+	   * loop now.  Otherwise, continue to loop until reaching a total of
+	   * $timeout seconds.
+	   */
+	  if (tmp.ch != -2 || SigWinch)
 	    goto gotkey;
 	  i -= ImapKeepalive;
 	  imap_keepalive ();
@@ -461,7 +465,7 @@ int km_dokey (int menu)
     if (tmp.op)
     {
       char *func = NULL;
-      struct binding_t *bindings;
+      const struct binding_t *bindings;
 
       /* is this a valid op for this menu? */
       if ((bindings = km_get_table (menu)) &&
@@ -536,7 +540,7 @@ int km_dokey (int menu)
   /* not reached */
 }
 
-static void create_bindings (struct binding_t *map, int menu)
+static void create_bindings (const struct binding_t *map, int menu)
 {
   int i;
 
@@ -545,10 +549,10 @@ static void create_bindings (struct binding_t *map, int menu)
       km_bindkey (map[i].seq, menu, map[i].op);
 }
 
-char *km_keyname (int c)
+static const char *km_keyname (int c)
 {
   static char buf[10];
-  char *p;
+  const char *p;
 
   if ((p = mutt_getnamebyvalue (c, KeyNames)))
     return p;
@@ -686,7 +690,7 @@ void init_extended_keys(void)
 
       if (keyname)
       {
-	char *s = tigetstr(keyname);
+        char *s = tigetstr((char *)keyname);
 	if (s && (long)(s) != -1)
 	{
 	  int code = key_defined(s);
@@ -862,7 +866,7 @@ static char *parse_keymap (int *menu, BUFFER *s, int maxmenus, int *nummenus, BU
   int i=0;
   char *p, *q;
 
-  memset (&buf, 0, sizeof (buf));
+  mutt_buffer_init (&buf);
 
   /* menu name */
   mutt_extract_token (&buf, s, 0);
@@ -907,7 +911,7 @@ error:
 }
 
 static int
-try_bind (char *key, int menu, char *func, struct binding_t *bindings)
+try_bind (char *key, int menu, char *func, const struct binding_t *bindings)
 {
   int i;
   
@@ -920,7 +924,7 @@ try_bind (char *key, int menu, char *func, struct binding_t *bindings)
   return (-1);
 }
 
-struct binding_t *km_get_table (int menu)
+const struct binding_t *km_get_table (int menu)
 {
   switch (menu)
   {
@@ -967,7 +971,7 @@ struct binding_t *km_get_table (int menu)
 /* bind menu-name '<key_sequence>' function-name */
 int mutt_parse_bind (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  struct binding_t *bindings = NULL;
+  const struct binding_t *bindings = NULL;
   char *key;
   int menu[sizeof(Menus)/sizeof(struct mapping_t)-1], r = 0, nummenus, i;
 
@@ -1068,7 +1072,7 @@ int mutt_parse_exec (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   int ops[128]; 
   int nops = 0;
-  struct binding_t *bindings = NULL;
+  const struct binding_t *bindings = NULL;
   char *function;
 
   if (!MoreArgs (s))
