@@ -56,8 +56,10 @@
 #include <getopt.h>
 #endif
 
-#ifdef HAVE_LIBIDN
+#ifdef HAVE_STRINGPREP_H
 #include <stringprep.h>
+#elif defined(HAVE_IDN_STRINGPREP_H)
+#include <idn/stringprep.h>
 #endif
 
 static const char *ReachingUs = N_("\
@@ -154,6 +156,24 @@ options:\n\
   exit (0);
 }
 
+extern const char cc_version[];
+extern const char cc_cflags[];
+extern const char configure_options[];
+
+static char *
+rstrip_in_place(char *s)
+{
+  char *p;
+
+  p = &s[strlen(s)];
+  if (p == s)
+    return s;
+  p--;
+  while (p >= s && (*p == '\n' || *p == '\r'))
+    *p-- = '\0';
+  return s;
+}
+
 static void show_version (void)
 {
   struct utsname uts;
@@ -193,6 +213,16 @@ static void show_version (void)
   printf ("\nhcache backend: %s", mutt_hcache_backend ());
 #endif
 
+  puts ("\n\nCompiler:");
+  rstrip_in_place((char *)cc_version);
+  puts (cc_version);
+
+  rstrip_in_place((char *)configure_options);
+  printf ("\nConfigure options: %s\n", configure_options);
+
+  rstrip_in_place((char *)cc_cflags);
+  printf ("\nCompilation CFLAGS: %s\n", cc_cflags);
+
   puts (_("\nCompile options:"));
 
 #ifdef DOMAIN
