@@ -119,7 +119,7 @@ int mutt_is_application_smime (BODY *);
 
 int mutt_signed_handler (BODY *, STATE *);
 
-int mutt_parse_crypt_hdr (char *, int, int);
+int mutt_parse_crypt_hdr (const char *, int, int);
 
 
 void convert_to_7bit (BODY *);
