@@ -992,16 +992,11 @@ int mutt_check_overwrite (const char *attname, const char *path,
     else if ((rc = mutt_yesorno (_("File is a directory, save under it?"), M_YES)) != M_YES)
       return (rc == M_NO) ? 1 : -1;
 
-    if (!attname || !attname[0])
-    {
-      tmp[0] = 0;
-      if (mutt_get_field (_("File under directory: "), tmp, sizeof (tmp),
-				      M_FILE | M_CLEAR) != 0 || !tmp[0])
-	return (-1);
-      mutt_concat_path (fname, path, tmp, flen);
-    }
-    else
-      mutt_concat_path (fname, path, mutt_basename (attname), flen);
+    strfcpy (tmp, mutt_basename (NONULL (attname)), sizeof (tmp));
+    if (mutt_get_field (_("File under directory: "), tmp, sizeof (tmp),
+                                    M_FILE | M_CLEAR) != 0 || !tmp[0])
+      return (-1);
+    mutt_concat_path (fname, path, tmp, flen);
   }
   
   if (*append == 0 && access (fname, F_OK) == 0)
@@ -1072,7 +1067,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
   prefix[0] = '\0';
   destlen--; /* save room for the terminal \0 */
-  wlen = (flags & M_FORMAT_ARROWCURSOR && option (OPTARROWCURSOR)) ? 3 : 0;
+  wlen = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
   col += wlen;
 
   if ((flags & M_FORMAT_NOFILTER) == 0)
@@ -1105,10 +1100,10 @@ void mutt_FormatString (char *dest,		/* output buffer */
       srccopy[n-1] = '\0';
 
       /* prepare BUFFERs */
-      srcbuf = mutt_buffer_from(NULL, srccopy);
+      srcbuf = mutt_buffer_from (srccopy);
       srcbuf->dptr = srcbuf->data;
-      word = mutt_buffer_init(NULL);
-      command = mutt_buffer_init(NULL);
+      word = mutt_buffer_new ();
+      command = mutt_buffer_new ();
 
       /* Iterate expansions across successive arguments */
       do {
@@ -1142,7 +1137,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
       col -= wlen;	/* reset to passed in value */
       wptr = dest;      /* reset write ptr */
-      wlen = (flags & M_FORMAT_ARROWCURSOR && option (OPTARROWCURSOR)) ? 3 : 0;
+      wlen = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
       if ((pid = mutt_create_filter(command->data, NULL, &filter, NULL)) != -1)
       {
 	int rc;
@@ -1312,7 +1307,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	  }
 	  else if (soft && pad < 0)
 	  {
-	    int offset = (flags & M_FORMAT_ARROWCURSOR && option (OPTARROWCURSOR)) ? 3 : 0;
+	    int offset = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
 	    /* \0-terminate dest for length computation in mutt_wstr_trunc() */
 	    *wptr = 0;
 	    /* make sure right part is at most as wide as display */
@@ -1643,28 +1638,22 @@ void mutt_sleep (short s)
   if (SleepTime > s)
     sleep (SleepTime);
   else if (s)
-    sleep (s);
+    sleep(s);
 }
 
-/*
- * Creates and initializes a BUFFER*. If passed an existing BUFFER*,
- * just initializes. Frees anything already in the buffer.
- *
- * Disregards the 'destroy' flag, which seems reserved for caller.
- * This is bad, but there's no apparent protocol for it.
- */
-BUFFER * mutt_buffer_init(BUFFER *b)
-{
-  if (!b)
-  {
-    b = safe_malloc(sizeof(BUFFER));
-    if (!b)
-      return NULL;
-  }
-  else
-  {
-    FREE(&b->data);
-  }
+/* creates and initializes a BUFFER */
+BUFFER *mutt_buffer_new(void) {
+  BUFFER *b;
+
+  b = safe_malloc(sizeof(BUFFER));
+
+  mutt_buffer_init(b);
+
+  return b;
+}
+
+/* initialize a new BUFFER */
+BUFFER *mutt_buffer_init (BUFFER *b) {
   memset(b, 0, sizeof(BUFFER));
   return b;
 }
@@ -1677,14 +1666,15 @@ BUFFER * mutt_buffer_init(BUFFER *b)
  * Disregards the 'destroy' flag, which seems reserved for caller.
  * This is bad, but there's no apparent protocol for it.
  */
-BUFFER * mutt_buffer_from(BUFFER *b, char *seed)
-{
+BUFFER *mutt_buffer_from (char *seed) {
+  BUFFER *b;
+
   if (!seed)
     return NULL;
 
-  b = mutt_buffer_init(b);
-  b->data = safe_strdup (seed);
-  b->dsize = mutt_strlen (seed);
+  b = mutt_buffer_new ();
+  b->data = safe_strdup(seed);
+  b->dsize = mutt_strlen(seed);
   b->dptr = (char *) b->data + b->dsize;
   return b;
 }
@@ -1749,7 +1739,7 @@ void mutt_buffer_free (BUFFER **p)
    FREE(p);		/* __FREE_CHECKED__ */
 }
 
-/* dynamically grows a BUFFER to accomodate s, in increments of 128 bytes.
+/* dynamically grows a BUFFER to accommodate s, in increments of 128 bytes.
  * Always one byte bigger than necessary for the null terminator, and
  * the buffer is always null-terminated */
 void mutt_buffer_add (BUFFER* buf, const char* s, size_t len)
@@ -1960,6 +1950,7 @@ void mutt_encode_path (char *dest, size_t dlen, const char *src)
 {
   char *p = safe_strdup (src);
   int rc = mutt_convert_string (&p, Charset, "utf-8", 0);
-  strfcpy (dest, rc == 0 ? p : src, dlen);
+  /* `src' may be NULL, such as when called from the pop3 driver. */
+  strfcpy (dest, (rc == 0) ? NONULL(p) : NONULL(src), dlen);
   FREE (&p);
 }
