@@ -299,7 +299,7 @@ resolve_color (struct line_t *lineInfo, int n, int cnt, int flags, int special,
 
   if (color != last_color)
   {
-    attrset (color);
+    ATTRSET (color);
     last_color = color;
   }
 }
@@ -1187,10 +1187,17 @@ static int format_line (struct line_t **lineInfo, int n, unsigned char *buf,
       last_special = special;
     }
 
-    if (IsWPrint (wc))
+    if (IsWPrint (wc) || (Charset_is_utf8 && wc == 0x00A0))
     {
       if (wc == ' ')
 	space = ch;
+      else if (Charset_is_utf8 && wc == 0x00A0)
+      {
+	/* Convert non-breaking space to normal space. The local variable
+	 * `space' is not set here so that the caller of this function won't
+	 * attempt to wrap at this character. */
+	wc = ' ';
+      }
       t = wcwidth (wc);
       if (col + t > wrap_cols)
 	break;
@@ -1453,7 +1460,7 @@ display_line (FILE *f, LOFF_T *last_pos, struct line_t **lineInfo, int n,
 #ifndef USE_SLANG_CURSES
   if (col == 0)
   {
-    SETCOLOR (MT_COLOR_NORMAL);
+    NORMAL_COLOR;
     addch (' ');
   }
 #endif
@@ -1475,10 +1482,7 @@ display_line (FILE *f, LOFF_T *last_pos, struct line_t **lineInfo, int n,
     else
       def_color = ColorDefs[ (*lineInfo)[m].type ];
 
-    attrset (def_color);
-#ifdef HAVE_BKGDSET
-    bkgdset (def_color | ' ');
-#endif
+    ATTRSET(def_color);
   }
 
   /* ncurses always wraps lines when you get to the right side of the
@@ -1496,10 +1500,7 @@ display_line (FILE *f, LOFF_T *last_pos, struct line_t **lineInfo, int n,
    * filled to the right margin.
    */
   if (flags & M_SHOWCOLOR)
-  {
-    SETCOLOR(MT_COLOR_NORMAL);
-    BKGDSET(MT_COLOR_NORMAL);
-  }
+    NORMAL_COLOR;
 
   /* build a return code */
   if (!(flags & M_SHOW))
@@ -1526,13 +1527,13 @@ upNLines (int nlines, struct line_t *info, int cur, int hiding)
   return cur;
 }
 
-static struct mapping_t PagerHelp[] = {
+static const struct mapping_t PagerHelp[] = {
   { N_("Exit"),	OP_EXIT },
   { N_("PrevPg"), OP_PREV_PAGE },
   { N_("NextPg"), OP_NEXT_PAGE },
   { NULL,	0 }
 };
-static struct mapping_t PagerHelpExtra[] = {
+static const struct mapping_t PagerHelpExtra[] = {
   { N_("View Attachm."), OP_VIEW_ATTACHMENTS },
   { N_("Del"), OP_DELETE },
   { N_("Reply"), OP_REPLY },
@@ -1636,7 +1637,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 
     if (redraw & REDRAW_FULL)
     {
-      SETCOLOR (MT_COLOR_NORMAL);
+      NORMAL_COLOR;
       /* clear() doesn't optimize screen redraws */
       move (0, 0);
       clrtobot ();
@@ -1674,7 +1675,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	SETCOLOR (MT_COLOR_STATUS);
 	move (helpoffset, 0);
 	mutt_paddstr (COLS, helpstr);
-	SETCOLOR (MT_COLOR_NORMAL);
+	NORMAL_COLOR;
       }
 
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
@@ -1707,7 +1708,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	  index->current = extra->hdr->virtual;
 	}
 
-	SETCOLOR (MT_COLOR_NORMAL);
+	NORMAL_COLOR;
 	index->offset  = indexoffset + (option (OPTSTATUSONTOP) ? 1 : 0);
 
 	index->pagelen = indexlen - 1;
@@ -1763,7 +1764,6 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
       } while (force_redraw);
 
       SETCOLOR (MT_COLOR_TILDE);
-      BKGDSET (MT_COLOR_TILDE);
       while (lines < bodylen)
       {
 	clrtoeol ();
@@ -1772,6 +1772,8 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	addch ('\n');
 	lines++;
       }
+      NORMAL_COLOR;
+
       /* We are going to update the pager status bar, so it isn't
        * necessary to reset to normal color now. */
 
@@ -1792,9 +1794,8 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	strfcpy(pager_progress_str, (topline == 0) ? "all" : "end", sizeof(pager_progress_str));
 
       /* print out the pager status bar */
+      move (statusoffset, 0);
       SETCOLOR (MT_COLOR_STATUS);
-      BKGDSET (MT_COLOR_STATUS);
-      CLEARLINE (statusoffset);
 
       if (IsHeader (extra) || IsMsgAttach (extra))
       {
@@ -1810,8 +1811,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	snprintf (bn, sizeof (bn), "%s (%s)", banner, pager_progress_str);
 	mutt_paddstr (COLS, bn);
       }
-      BKGDSET (MT_COLOR_NORMAL);
-      SETCOLOR (MT_COLOR_NORMAL);
+      NORMAL_COLOR;
     }
 
     if ((redraw & REDRAW_INDEX) && index)
@@ -1825,10 +1825,8 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
  
       move (indexoffset + (option (OPTSTATUSONTOP) ? 0 : (indexlen - 1)), 0);
       SETCOLOR (MT_COLOR_STATUS);
-      BKGDSET (MT_COLOR_STATUS);
       mutt_paddstr (COLS, buffer);
-      SETCOLOR (MT_COLOR_NORMAL);
-      BKGDSET (MT_COLOR_NORMAL);
+      NORMAL_COLOR;
     }
 
     redraw = 0;
@@ -1929,6 +1927,15 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	ch = -1;
 	break;
 
+      case OP_QUIT:
+	if (query_quadoption (OPT_QUIT, _("Quit Mutt?")) == M_YES)
+	{
+	  /* avoid prompting again in the index menu */
+	  set_quadoption (OPT_QUIT, M_YES);
+	  ch = -1;
+	}
+	break;
+
       case OP_NEXT_PAGE:
 	if (lineInfo[curline].offset < sb.st_size-1)
 	{
