@@ -37,7 +37,7 @@
 #include <string.h>
 #include <sys/stat.h>
 
-static struct mapping_t PostponeHelp[] = {
+static const struct mapping_t PostponeHelp[] = {
   { N_("Exit"),  OP_EXIT },
   { N_("Del"),   OP_DELETE },
   { N_("Undel"), OP_UNDELETE },
@@ -71,6 +71,7 @@ int mutt_num_postponed (int force)
 
   if (mutt_strcmp (Postponed, OldPostponed))
   {
+    FREE (&OldPostponed);
     OldPostponed = safe_strdup (Postponed);
     LastModify = 0;
     force = 1;
@@ -232,7 +233,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   LIST *tmp;
   LIST *last = NULL;
   LIST *next;
-  char *p;
+  const char *p;
   int opt_delete;
 
   if (!Postponed)
@@ -295,8 +296,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       {
 	/* if a mailbox is currently open, look to see if the orignal message
 	   the user attempted to reply to is in this mailbox */
-	p = tmp->data + 18;
-	SKIPWS (p);
+	p = skip_email_wsp(tmp->data + 18);
 	if (!ctx->id_hash)
 	  ctx->id_hash = mutt_make_id_hash (ctx);
 	*cur = hash_find (ctx->id_hash, p);
@@ -316,8 +316,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
     }
     else if (ascii_strncasecmp ("X-Mutt-Fcc:", tmp->data, 11) == 0)
     {
-      p = tmp->data + 11;
-      SKIPWS (p);
+      p = skip_email_wsp(tmp->data + 11);
       strfcpy (fcc, p, fcclen);
       mutt_pretty_mailbox (fcc, fcclen);
 
@@ -330,6 +329,12 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
+     /* note that x-mutt-fcc was present.  we do this because we want to add a
+      * default fcc if the header was missing, but preserve the request of the
+      * user to not make a copy if the header field is present, but empty.
+      * see http://dev.mutt.org/trac/ticket/3653
+      */
+      code |= SENDPOSTPONEDFCC;
     }
     else if ((WithCrypto & APPLICATION_PGP)
              && (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
@@ -404,7 +409,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 
 
 
-int mutt_parse_crypt_hdr (char *p, int set_signas, int crypt_app)
+int mutt_parse_crypt_hdr (const char *p, int set_signas, int crypt_app)
 {
   char smime_cryptalg[LONG_STRING] = "\0";
   char sign_as[LONG_STRING] = "\0", *q;
@@ -413,7 +418,7 @@ int mutt_parse_crypt_hdr (char *p, int set_signas, int crypt_app)
   if (!WithCrypto)
     return 0;
 
-  SKIPWS (p);
+  p = skip_email_wsp(p);
   for (; *p; p++)
   {
 
@@ -546,8 +551,15 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   newhdr->content->length = hdr->content->length;
   mutt_parse_part (fp, newhdr->content);
 
-  FREE (&newhdr->env->message_id);
-  FREE (&newhdr->env->mail_followup_to); /* really? */
+  /* If message_id is set, then we are resending a message and don't want
+   * message_id or mail_followup_to. Otherwise, we are resuming a
+   * postponed message, and want to keep the mail_followup_to.
+   */
+  if (newhdr->env->message_id != NULL)
+  {
+    FREE (&newhdr->env->message_id);
+    FREE (&newhdr->env->mail_followup_to);
+  }
 
   /* decrypt pgp/mime encoded messages */
 
