@@ -65,7 +65,7 @@ extern char RFC822Specials[];
 
 const char MimeSpecials[] = "@.,;:<>[]\\\"()?/= \t";
 
-char B64Chars[64] = {
+const char B64Chars[64] = {
   'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
   'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
   'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
@@ -1664,7 +1664,7 @@ static int fold_one_header (FILE *fp, const char *tag, const char *value,
     /* find the next word and place it in `buf'. it may start with
      * whitespace we can fold before */
     next = find_word (p);
-    l = MIN(sizeof (buf), next - p);
+    l = MIN(sizeof (buf) - 1, next - p);
     memcpy (buf, p, l);
     buf[l] = 0;
 
@@ -1799,7 +1799,7 @@ static int write_one_header (FILE *fp, int pfxw, int max, int wraplen,
   else
   {
     t = strchr (start, ':');
-    if (t > end)
+    if (!t || t > end)
     {
       dprint (1, (debugfile, "mwoh: warning: header not in "
 		  "'key: value' format!\n"));
@@ -1813,8 +1813,8 @@ static int write_one_header (FILE *fp, int pfxw, int max, int wraplen,
     else
     {
       tagbuf = mutt_substrdup (start, t);
-      ++t; /* skip over the colon separating the header field name and value */
-      SKIPWS(t); /* skip over any leading whitespace */
+      /* skip over the colon separating the header field name and value */
+      t = skip_email_wsp(t + 1);
       valbuf = mutt_substrdup (t, end);
     }
     dprint(4,(debugfile,"mwoh: buf[%s%s] too long, "
@@ -2036,7 +2036,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
 
       *p = '\0';
 
-      p++; SKIPWS (p);
+      p = skip_email_wsp(p + 1);
       if (!*p)
       {
 	*q = ':';
@@ -2080,7 +2080,7 @@ static void encode_headers (LIST *h)
       continue;
 
     i = p - h->data;
-    ++p; SKIPWS (p);
+    p = skip_email_wsp(p + 1);
     tmp = safe_strdup (p);
 
     if (!tmp)
@@ -2139,7 +2139,7 @@ char *mutt_gen_msgid (void)
   return (safe_strdup (buf));
 }
 
-static RETSIGTYPE alarm_handler (int sig)
+static void alarm_handler (int sig)
 {
   SigAlrm = 1;
 }
@@ -2344,6 +2344,13 @@ mutt_invoke_sendmail (ADDRESS *from,	/* the sender */
   size_t argslen = 0, argsmax = 0;
   int i;
 
+  /* ensure that $sendmail is set to avoid a crash. http://dev.mutt.org/trac/ticket/3548 */
+  if (!s)
+  {
+    mutt_error(_("$sendmail must be set in order to send mail."));
+    return -1;
+  }
+
   ps = s;
   i = 0;
   while ((ps = strtok (ps, " ")))
@@ -2566,7 +2573,7 @@ static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *r
 
 int mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
 {
-  ADDRESS *from;
+  ADDRESS *from, *resent_to;
   const char *fqdn = mutt_fqdn (1);
   char resent_from[STRING];
   int ret;
@@ -2598,8 +2605,17 @@ int mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
   }
   rfc822_write_address (resent_from, sizeof (resent_from), from, 0);
 
-  ret = _mutt_bounce_message (fp, h, to, resent_from, from);
+  /*
+   * prepare recipient list. idna conversion appears to happen before this
+   * function is called, since the user receives confirmation of the address
+   * list being bounced to.
+   */
+  resent_to = rfc822_cpy_adr(to, 0);
+  rfc2047_encode_adrlist(resent_to, "Resent-To");
 
+  ret = _mutt_bounce_message (fp, h, resent_to, resent_from, from);
+
+  rfc822_free_address (&resent_to);
   rfc822_free_address (&from);
 
   return ret;
