@@ -357,7 +357,7 @@ char* smime_ask_for_key (char *prompt, char *mailbox, short public)
   char index_file[_POSIX_PATH_MAX];
   FILE *index;
   char buf[LONG_STRING];
-  char fields[5][STRING];
+  char fields[5][STRING+1]; /* +1 due to use of fscanf() below. the max field width does not include the null terminator (see http://dev.mutt.org/trac/ticket/3636) */
   int numFields, hash_suffix, done, cur; /* The current entry */
   MUTTMENU* menu;
   unsigned int hash;
@@ -470,7 +470,7 @@ char *smime_get_field_from_db (char *mailbox, char *query, short public, short m
   int addr_len, query_len, found = 0, ask = 0, choice = 0;
   char cert_path[_POSIX_PATH_MAX];
   char buf[LONG_STRING], prompt[STRING];
-  char fields[5][STRING];
+  char fields[5][STRING+1]; /* +1 due to use of fscanf() below. the max field width does not include the null terminator (see http://dev.mutt.org/trac/ticket/3636) */
   char key[STRING];  
   int numFields;
   struct stat info;
@@ -635,7 +635,7 @@ char *smime_get_field_from_db (char *mailbox, char *query, short public, short m
 
 /* 
    This sets the '*ToUse' variables for an upcoming decryption, where
-   the reuquired key is different from SmimeDefaultKey.
+   the required key is different from SmimeDefaultKey.
 */
 
 void _smime_getkeys (char *mailbox)
@@ -1546,7 +1546,7 @@ int smime_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
   pid_t thepid;
   int badsig = -1;
 
-  long tmpoffset = 0;
+  LOFF_T tmpoffset = 0;
   size_t tmplength = 0;
   int origType = sigbdy->type;
   char *savePrefix = NULL;
@@ -1660,7 +1660,6 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
 {
   int len=0;
   int c;
-  long last_pos;
   char buf[HUGE_STRING];
   char outfile[_POSIX_PATH_MAX], errfile[_POSIX_PATH_MAX];
   char tmpfname[_POSIX_PATH_MAX];
@@ -1701,7 +1700,6 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
   }
 
   fseeko (s->fpin, m->offset, 0);
-  last_pos = m->offset;
 
   mutt_copy_bytes (s->fpin, tmpfp,  m->length);
 
@@ -1865,7 +1863,7 @@ int smime_decrypt_mime (FILE *fpin, FILE **fpout, BODY *b, BODY **cur)
 
   char tempfile[_POSIX_PATH_MAX];
   STATE s;
-  long tmpoffset = b->offset;
+  LOFF_T tmpoffset = b->offset;
   size_t tmplength = b->length;
   int origType = b->type;
   FILE *tmpfp=NULL;
